@@ -1,6 +1,4 @@
-import json
 import logging
-from pathlib import Path
 
 import packaging.version
 import requests
@@ -40,7 +38,6 @@ from .db.annotation_classes import (
     delete_annotation_class, download_annotation_classes_json,
     get_annotation_class_metadata, search_annotation_classes
 )
-from .db.clone_project import clone_project
 from .db.exports import (
     download_export, get_export_metadata, get_exports, prepare_export
 )
@@ -52,26 +49,34 @@ from .db.images import (
     create_fuse_image, delete_image, download_image, download_image_annotations,
     download_image_preannotations, get_image_annotations, get_image_bytes,
     get_image_metadata, get_image_preannotations, search_images,
-    set_image_annotation_status, upload_image_annotations
+    search_images_all_folders, set_image_annotation_status,
+    set_images_annotation_statuses, upload_image_annotations,
+    get_project_root_folder_id
+)
+from .db.project_api import (
+    create_folder, delete_folders, get_folder_metadata,
+    get_project_and_folder_metadata, rename_folder, search_folders
 )
 from .db.project_images import (
-    assign_images, copy_image, move_image, pin_image, upload_image_to_project
+    assign_images, copy_image, copy_images, delete_images, move_image,
+    move_images, pin_image, upload_image_to_project, assign_folder,
+    unassign_folder, unassign_images
 )
-from .db.project_metadata import get_project_metadata
 from .db.projects import (
-    create_project, create_project_from_metadata, delete_project,
+    clone_project, create_project, create_project_from_metadata, delete_project,
     get_project_default_image_quality_in_editor, get_project_image_count,
-    get_project_settings, get_project_workflow, rename_project,
-    set_project_default_image_quality_in_editor, set_project_settings,
-    set_project_workflow, share_project, unshare_project,
+    get_project_metadata, get_project_settings, get_project_workflow,
+    rename_project, set_project_default_image_quality_in_editor,
+    set_project_settings, set_project_workflow, share_project, unshare_project,
     upload_annotations_from_folder_to_project,
     upload_images_from_azure_blob_to_project,
     upload_images_from_folder_to_project,
     upload_images_from_google_cloud_to_project,
     upload_images_from_public_urls_to_project,
     upload_images_from_s3_bucket_to_project, upload_images_to_project,
-    upload_preannotations_from_folder_to_project, upload_video_to_project,
-    upload_videos_from_folder_to_project
+    attach_image_urls_to_project, upload_preannotations_from_folder_to_project,
+    upload_video_to_project, upload_videos_from_folder_to_project,
+    attach_video_urls_to_project
 )
 from .db.search_projects import search_projects
 from .db.teams import (
@@ -86,15 +91,14 @@ from .exceptions import (
     SANonExistingProjectNameException
 )
 from .input_converters.conversion import (
-    coco_split_dataset, convert_project_type, export_annotation,
-    import_annotation, convert_json_version
+    coco_split_dataset, convert_json_version, convert_project_type,
+    export_annotation, import_annotation
 )
 from .ml.ml_funcs import (
     delete_model, download_model, plot_model_metrics, run_prediction,
     run_segmentation, run_training, stop_model_training
 )
 from .ml.ml_models import search_models
-
 from .old_to_new_format_convertor import update_json_format
 from .version import __version__
 
