@@ -1,6 +1,8 @@
 import os
 import json
 import logging
+import sys
+import time
 from pathlib import Path
 
 import requests
@@ -9,6 +11,7 @@ import urllib3
 
 from .exceptions import SABaseException
 from .version import __version__
+from .mixp.app import mp, get_default
 
 logger = logging.getLogger("superannotate-python-sdk")
 
@@ -24,10 +27,13 @@ class API:
         self._default_headers = None
         self._main_endpoint = None
         self.team_id = None
+        self.user_id = None
+        self.team_name = None
         if API.__instance is not None:
             raise SABaseException(0, "API class is a singleton!")
         API.__instance = self
         self._authenticated = False
+        self.init()
 
     def init(self, config_location=None):
         if config_location is None:
@@ -46,7 +52,7 @@ class API:
             self._api_config = json.load(open(config_location))
 
             try:
-                self._token = self._api_config["token"]
+                self._token = self._api_config["token"].strip()
             except KeyError:
                 raise SABaseException(
                     0,
@@ -78,15 +84,14 @@ class API:
                 self._verify = self._api_config["ssl_verify"]
             self._session = None
             self._authenticated = True
+
             response = self.send_request(
                 req_type='GET',
-                path='/projects',
-                params={
-                    'team_id': str(self.team_id),
-                    'offset': 0,
-                    'limit': 1
-                }
+                path=f'/team/{self.team_id}',
             )
+            self.user_id = response.json().get('creator_id', None)
+            self.team_name = response.json().get('name', None)
+
             if not self._verify:
                 urllib3.disable_warnings(
                     urllib3.exceptions.InsecureRequestWarning
@@ -102,6 +107,11 @@ class API:
                 raise SABaseException(
                     0, "Couldn't reach superannotate " + response.text
                 )
+            if "pytest" not in sys.modules:
+                mp.track(
+                    self.user_id, "SDK init",
+                    get_default(self.team_name, self.user_id)
+                )
         except SABaseException:
             self._authenticated = False
             self._session = None
@@ -123,6 +133,10 @@ class API:
             )
         url = self._main_endpoint + path
 
+        if params is not None:
+            for key, value in params.items():
+                if isinstance(value, str):
+                    params[key] = value.replace("\\", "\\\\")
         req = requests.Request(
             method=req_type, url=url, json=json_req, params=params
         )
@@ -138,11 +152,11 @@ class API:
             total=5,
             read=5,
             connect=5,
-            backoff_factor=0.3,
+            backoff_factor=2,
             # use on any request type
             method_whitelist=False,
             # force retry on those status responses
-            status_forcelist=(501, 502, 503, 504, 505, 506, 507, 508, 510, 511),
+            status_forcelist=(501, 502, 503, 504, 505, 506, 507, 508, 510, 511, 403, 400),
             raise_on_status=False
         )
         adapter = requests.adapters.HTTPAdapter(
