@@ -4,15 +4,22 @@ import logging
 import os
 import sys
 import time
-from tqdm import tqdm
 from pathlib import Path
-from enum import IntEnum
+
 import numpy as np
 from PIL import Image
+from tqdm import tqdm
 
 logger = logging.getLogger("superannotate-python-sdk")
 
-_PROJECT_TYPES = {"Vector": 1, "Pixel": 2}
+DEFAULT_IMAGE_EXTENSIONS = ("jpg", "jpeg", "png", "tif", "tiff", "webp", "bmp")
+DEFAULT_FILE_EXCLUDE_PATTERNS = ("___save.png", "___fuse.png")
+
+DEFAULT_VIDEO_EXTENSIONS = ("mp4", "avi", "mov", "webm", "flv", "mpg", "ogg")
+
+SPECIAL_CHARACTERS_IN_PROJECT_FOLDER_NAMES = set('/\\:*?"<>|')
+
+_PROJECT_TYPES = {"Vector": 1, "Pixel": 2, "Video": 3}
 
 _ANNOTATION_STATUSES = {
     "NotStarted": 1,
@@ -22,6 +29,10 @@ _ANNOTATION_STATUSES = {
     "Completed": 5,
     "Skipped": 6
 }
+
+_UPLOAD_STATES_STR_TO_CODES = {"Initial": 1, "Basic": 2, "External": 3}
+_UPLOAD_STATES_CODES_TO_STR = {1: "Initial", 2: "Basic", 3: "External"}
+
 _USER_ROLES = {"Admin": 2, "Annotator": 3, "QA": 4, "Customer": 5, "Viewer": 6}
 _AVAILABLE_SEGMENTATION_MODELS = ['autonomous', 'generic']
 _MODEL_TRAINING_STATUSES = {
@@ -40,6 +51,54 @@ _PREDICTION_SEGMENTATION_STATUSES = {
     "Failed": 4
 }
 
+_MODEL_TRAINING_TASKS = {
+    "Instance Segmentation for Pixel Projects": "instance_segmentation_pixel",
+    "Instance Segmentation for Vector Projects": "instance_segmentation_vector",
+    "Keypoint Detection for Vector Projects": "keypoint_detection_vector",
+    "Object Detection for Vector Projects": "object_detection_vector",
+    "Semantic Segmentation for Pixel Projects": "semantic_segmentation_pixel"
+}
+
+VIDEO_DEPRICATED_FUNCTIONS = [
+    "upload_images_from_folder_to_project",
+    "get_image_metadata",
+    "search_images",
+    "upload_images_to_project",
+    "upload_annotations_from_folder_to_project",
+    "upload_image_annotations",
+    "download_image",
+    "download_image_annotations",
+    "get_image_annotations",
+    "set_image_annotation_status",
+    "aggregate_annotations_as_df",
+    "attach_image_urls_to_project",
+    "clone_project",
+    "copy_image",
+    "export_annotation",
+    "upload_image_to_project",
+    "upload_video_to_project",
+    "add_annotation_bbox_to_image",
+    "assign_images",
+    "delete_images",
+    "get_project_image_count",
+    "set_project_workflow",
+    "upload_preannotations_from_folder_to_project",
+    "upload_videos_from_folder_to_project",
+    "add_annotation_comment_to_image",
+    "add_annotation_point_to_image",
+    "benchmark",
+    "class_distribution",
+    "consensus",
+    "convert_project_type",
+    "copy_images",
+    "get_project_workflow",
+    "move_image",
+    "move_images",
+    "set_images_annotation_statuses",
+    "set_project_default_image_quality_in_editor",
+    "upload_images_from_google_cloud_to_project",
+]
+
 
 def prediction_segmentation_status_from_str_to_int(status):
     return _PREDICTION_SEGMENTATION_STATUSES[status]
@@ -65,13 +124,15 @@ def model_training_status_str_to_int(project_status):
 
 def image_path_to_annotation_paths(image_path, project_type):
     image_path = Path(image_path)
-    postfix_json = '___objects.json' if project_type == "Vector" else '___pixel.json'
-    postfix_mask = '___save.png'
     if project_type == "Vector":
-        return (image_path.parent / (image_path.name + postfix_json), )
+        return (
+            image_path.parent /
+            get_annotation_json_name(image_path.name, project_type),
+        )
     return (
-        image_path.parent / (image_path.name + postfix_json),
-        image_path.parent / (image_path.name + postfix_mask)
+        image_path.parent /
+        get_annotation_json_name(image_path.name, project_type),
+        image_path.parent / get_annotation_png_name(image_path.name)
     )
 
 
@@ -88,10 +149,11 @@ def project_type_int_to_str(project_type):
     :return: 'Vector' or 'Pixel'
     :rtype: str
     """
+    if project_type not in _PROJECT_TYPES.values():
+        raise RuntimeError("NA Project type")
     for k, v in _PROJECT_TYPES.items():
         if v == project_type:
             return k
-    raise RuntimeError("NA Project type")
 
 
 def user_role_str_to_int(user_role):
@@ -109,6 +171,14 @@ def annotation_status_str_to_int(annotation_status):
     return _ANNOTATION_STATUSES[annotation_status]
 
 
+def upload_state_str_to_int(upload_state):
+    return _UPLOAD_STATES_STR_TO_CODES[upload_state]
+
+
+def upload_state_int_to_str(upload_state):
+    return _UPLOAD_STATES_CODES_TO_STR[upload_state]
+
+
 def annotation_status_int_to_str(annotation_status):
     """Converts metadata annotation_status int value to a string
 
@@ -150,9 +220,6 @@ def rename_kwargs(func_name, kwargs, aliases):
             kwargs[new] = kwargs.pop(alias)
 
 
-logger = logging.getLogger('httplogger')
-
-
 def hex_to_rgb(hex_string):
     """Converts HEX values to RGB values
     """
@@ -296,16 +363,8 @@ MAX_IMAGE_RESOLUTION = {
 }  # Resolution limit
 
 
-def process_api_response(data):
-    logger.info("Maintenance operations are being performed")
-    if 'metadata' not in data:
-        return data
-
-    return data['data']
-
-
 def tqdm_converter(
-    total_num, images_converted, images_not_converted, finish_event
+        total_num, images_converted, images_not_converted, finish_event
 ):
     with tqdm(total=total_num) as pbar:
         while True:
@@ -316,3 +375,14 @@ def tqdm_converter(
             else:
                 pbar.update(total_num - pbar.n)
                 break
+
+
+def get_annotation_json_name(image_name, project_type):
+    if project_type == "Vector":
+        return image_name + "___objects.json"
+    else:
+        return image_name + "___pixel.json"
+
+
+def get_annotation_png_name(image_name):
+    return image_name + "___save.png"
