@@ -39,10 +39,10 @@ def image_consensus(df, image_name, annot_type):
 
     """
     image_df = df[df["imageName"] == image_name]
-    all_projects = list(set(df["project"]))
+    all_projects = list(set(df["folderName"]))
     column_names = [
         "creatorEmail", "imageName", "instanceId", "area", "className",
-        "attributes", "projectName", "score"
+        "attributes", "folderName", "score"
     ]
     instance_id = 0
     image_data = {}
@@ -52,8 +52,8 @@ def image_consensus(df, image_name, annot_type):
     projects_shaply_objs = {}
     # generate shapely objects of instances
     for _, row in image_df.iterrows():
-        if row["project"] not in projects_shaply_objs:
-            projects_shaply_objs[row["project"]] = []
+        if row["folderName"] not in projects_shaply_objs:
+            projects_shaply_objs[row["folderName"]] = []
         inst_data = row["meta"]
         if annot_type == 'bbox':
             inst_coords = inst_data["points"]
@@ -69,7 +69,7 @@ def image_consensus(df, image_name, annot_type):
         elif annot_type == 'point':
             inst = Point(inst_data["x"], inst_data["y"])
         if inst.is_valid:
-            projects_shaply_objs[row["project"]].append(
+            projects_shaply_objs[row["folderName"]].append(
                 (
                     inst, row["className"], row["creatorEmail"],
                     row["attributes"]
@@ -80,32 +80,43 @@ def image_consensus(df, image_name, annot_type):
                 "Invalid %s instance occured, skipping to the next one.",
                 annot_type
             )
+    visited_instances = {}
+    for proj, instances in projects_shaply_objs.items():
+        visited_instances[proj] = [False] * len(instances)
 
     # match instances
     for curr_proj, curr_proj_instances in projects_shaply_objs.items():
-        for curr_inst_data in curr_proj_instances:
+        for curr_id, curr_inst_data in enumerate(curr_proj_instances):
             curr_inst, curr_class, _, _ = curr_inst_data
+            if visited_instances[curr_proj][curr_id] == True:
+                continue
             max_instances = []
             for other_proj, other_proj_instances in projects_shaply_objs.items(
             ):
                 if curr_proj == other_proj:
                     max_instances.append((curr_proj, *curr_inst_data))
-                    projects_shaply_objs[curr_proj].remove(curr_inst_data)
+                    visited_instances[curr_proj][curr_id] = True
                 else:
                     if annot_type in ['polygon', 'bbox']:
                         max_score = 0
                     else:
                         max_score = float('-inf')
                     max_inst_data = None
-                    for other_inst_data in other_proj_instances:
+                    max_inst_id = -1
+                    for other_id, other_inst_data in enumerate(
+                        other_proj_instances
+                    ):
                         other_inst, other_class, _, _ = other_inst_data
+                        if visited_instances[other_proj][other_id] == True:
+                            continue
                         score = instance_consensus(curr_inst, other_inst)
                         if score > max_score and other_class == curr_class:
                             max_score = score
                             max_inst_data = other_inst_data
+                            max_inst_id = other_id
                     if max_inst_data is not None:
                         max_instances.append((other_proj, *max_inst_data))
-                        projects_shaply_objs[other_proj].remove(max_inst_data)
+                        visited_instances[other_proj][max_inst_id] = True
             if len(max_instances) == 1:
                 image_data["creatorEmail"].append(max_instances[0][3])
                 image_data["attributes"].append(max_instances[0][4])
@@ -113,7 +124,7 @@ def image_consensus(df, image_name, annot_type):
                 image_data["imageName"].append(image_name)
                 image_data["instanceId"].append(instance_id)
                 image_data["className"].append(max_instances[0][2])
-                image_data["projectName"].append(max_instances[0][0])
+                image_data["folderName"].append(max_instances[0][0])
                 image_data["score"].append(0)
             else:
                 for curr_match_data in max_instances:
@@ -130,7 +141,7 @@ def image_consensus(df, image_name, annot_type):
                     image_data["imageName"].append(image_name)
                     image_data["instanceId"].append(instance_id)
                     image_data["className"].append(curr_match_data[2])
-                    image_data["projectName"].append(curr_match_data[0])
+                    image_data["folderName"].append(curr_match_data[0])
                     image_data["score"].append(
                         proj_cons / (len(all_projects) - 1)
                     )
@@ -156,10 +167,10 @@ def consensus_plot(consensus_df, projects):
     #project-wise boxplot
     project_box_fig = px.box(
         plot_data,
-        x="projectName",
+        x="folderName",
         y="score",
         points="all",
-        color="projectName",
+        color="folderName",
         color_discrete_sequence=px.colors.qualitative.Dark24
     )
     project_box_fig.show()
@@ -171,12 +182,12 @@ def consensus_plot(consensus_df, projects):
         y="score",
         color="className",
         symbol="creatorEmail",
-        facet_col="projectName",
+        facet_col="folderName",
         color_discrete_sequence=px.colors.qualitative.Dark24,
         hover_data={
             "className": False,
             "imageName": True,
-            "projectName": False,
+            "folderName": False,
             "area": False,
             "score": False
         },
