@@ -11,17 +11,20 @@ from ..exceptions import (
     SANonExistingAnnotationClassNameException
 )
 from .project_api import get_project_metadata_bare
+from ..mixp.decorators import Trackable
+from .utils import get_templates_mapping
 
 logger = logging.getLogger("superannotate-python-sdk")
 
 _api = API.get_instance()
 
 
+@Trackable
 def create_annotation_class(project, name, color, attribute_groups=None):
     """Create annotation class in project
 
-    :param project: project name or metadata of the project
-    :type project: str or dict
+    :param project: project name
+    :type project: str
     :param name: name for the class
     :type name: str
     :param color: RGB hex color value, e.g., "#FFFFAA"
@@ -79,11 +82,12 @@ def create_annotation_class(project, name, color, attribute_groups=None):
     return new_class
 
 
+@Trackable
 def delete_annotation_class(project, annotation_class):
     """Deletes annotation class from project
 
-    :param project: project name or metadata of the project
-    :type project: str or dict
+    :param project: project name
+    :type project: str
     :param project: annotation class name or  metadata
     :type project: str or dict
     """
@@ -113,14 +117,15 @@ def delete_annotation_class(project, annotation_class):
         )
 
 
+@Trackable
 def create_annotation_classes_from_classes_json(
     project, classes_json, from_s3_bucket=None
 ):
     """Creates annotation classes in project from a SuperAnnotate format
     annotation classes.json.
 
-    :param project: project name or metadata of the project
-    :type project: str or dict
+    :param project: project name
+    :type project: str
     :param classes_json: JSON itself or path to the JSON file
     :type classes_json: list or Pathlike (str or Path)
     :param from_s3_bucket: AWS S3 bucket to use. If None then classes_json is in local filesystem
@@ -200,11 +205,12 @@ def create_annotation_classes_from_classes_json(
     return res
 
 
+@Trackable
 def search_annotation_classes(project, name_prefix=None):
     """Searches annotation classes by name_prefix (case-insensitive)
 
-    :param project: project name or metadata of the project
-    :type project: str or dict
+    :param project: project name
+    :type project: str
     :param name_prefix: name prefix for search. If None all annotation classes
      will be returned
     :type name_prefix: str
@@ -239,11 +245,12 @@ def search_annotation_classes(project, name_prefix=None):
     return result_list
 
 
+@Trackable
 def get_annotation_class_metadata(project, annotation_class_name):
     """Returns annotation class metadata
 
-    :param project: project name or metadata of the project
-    :type project: str or dict
+    :param project: project name
+    :type project: str
     :param annotation_class_name: annotation class name
     :type annotation_class_name: str
 
@@ -272,11 +279,12 @@ def get_annotation_class_metadata(project, annotation_class_name):
         )
 
 
+@Trackable
 def download_annotation_classes_json(project, folder):
     """Downloads project classes.json to folder
 
-    :param project: project name or metadata of the project
-    :type project: str or dict
+    :param project: project name
+    :type project: str
     :param folder: folder to download to
     :type folder: Pathlike (str or Path)
 
@@ -296,21 +304,52 @@ def download_annotation_classes_json(project, folder):
 
 
 def fill_class_and_attribute_names(annotations_json, annotation_classes_dict):
+    if "instances" not in annotations_json:
+        return
     for r in annotations_json["instances"]:
-        if "classId" in r and r["classId"] in annotation_classes_dict:
-            r["className"] = annotation_classes_dict[r["classId"]]["name"]
-            if "attributes" in r:
-                for attribute in r["attributes"]:
-                    if "groupId" in attribute and "id" in attribute:
-                        attribute["groupName"] = annotation_classes_dict[
-                            r["classId"]]["attribute_groups"][
-                                attribute["groupId"]]["name"]
-                        attribute["name"] = annotation_classes_dict[
-                            r["classId"]]["attribute_groups"][attribute[
-                                "groupId"]]["attributes"][attribute["id"]]
+        if "classId" not in r or r["classId"] not in annotation_classes_dict:
+            continue
+        r["className"] = annotation_classes_dict[r["classId"]]["name"]
+        if "attributes" not in r:
+            continue
+        for attribute in r["attributes"]:
+            if "groupId" not in attribute or "id" not in attribute:
+                continue
+            if attribute["groupId"] not in annotation_classes_dict[
+                r["classId"]]["attribute_groups"]:
+                continue
+            attribute["groupName"] = annotation_classes_dict[
+                r["classId"]]["attribute_groups"][attribute["groupId"]]["name"]
+            if attribute["id"] not in annotation_classes_dict[r["classId"]][
+                "attribute_groups"][attribute["groupId"]]["attributes"]:
+                continue
+            attribute["name"] = annotation_classes_dict[
+                r["classId"]]["attribute_groups"][
+                    attribute["groupId"]]["attributes"][attribute["id"]]
 
 
 def fill_class_and_attribute_ids(annotation_json, annotation_classes_dict):
+    if "instances" not in annotation_json:
+        return
+
+    unknown_classes = {}
+    for ann in [i for i in annotation_json["instances"] if "className" in i]:
+        if "className" not in ann:
+            continue
+        annotation_class_name = ann["className"]
+        if not annotation_class_name in annotation_classes_dict:
+            if annotation_class_name not in unknown_classes:
+                class_num = -(len(unknown_classes) + 1)
+                unknown_classes[annotation_class_name] = {
+                    'id': class_num,
+                    'attribute_groups': {}
+                }
+    annotation_classes_dict = {**annotation_classes_dict, **unknown_classes}
+    templates_map = get_templates_mapping()
+    for ann in (
+        i for i in annotation_json["instances"] if i.get('type', None) == 'template'
+    ):
+        ann['templateId'] = templates_map.get(ann.get('templateName', ''), -1)
     for ann in annotation_json["instances"]:
         if "className" not in ann:
             logger.warning("No className in annotation instance")
@@ -324,28 +363,27 @@ def fill_class_and_attribute_ids(annotation_json, annotation_classes_dict):
         class_id = annotation_classes_dict[annotation_class_name]["id"]
         ann["classId"] = class_id
         for attribute in ann["attributes"]:
-            if attribute["groupName"] in annotation_classes_dict[
+            if attribute["groupName"] not in annotation_classes_dict[
                 annotation_class_name]["attribute_groups"]:
-                attribute["groupId"] = annotation_classes_dict[
-                    annotation_class_name]["attribute_groups"][
-                        attribute["groupName"]]["id"]
-            else:
                 logger.warning(
                     "Couldn't find annotation group %s", attribute["groupName"]
                 )
                 continue
-            if attribute["name"] in annotation_classes_dict[
+            attribute["groupId"] = annotation_classes_dict[
+                annotation_class_name]["attribute_groups"][
+                    attribute["groupName"]]["id"]
+            if attribute["name"] not in annotation_classes_dict[
                 annotation_class_name]["attribute_groups"][
                     attribute["groupName"]]["attributes"]:
-                attribute["id"] = annotation_classes_dict[
-                    annotation_class_name]["attribute_groups"][
-                        attribute["groupName"]]["attributes"][attribute["name"]]
-            else:
                 logger.warning(
                     "Couldn't find annotation name %s in annotation group %s",
                     attribute["name"], attribute["groupName"]
                 )
                 del attribute["groupId"]
+                continue
+            attribute["id"] = annotation_classes_dict[annotation_class_name][
+                "attribute_groups"][attribute["groupName"]]["attributes"][
+                    attribute["name"]]
 
 
 def check_annotation_json(annotation_json):
