@@ -6,16 +6,20 @@ import time
 import zipfile
 from datetime import datetime
 from pathlib import Path
+import shutil
 
 import boto3
 import requests
 from tqdm import tqdm
 
 from ..api import API
-from ..common import annotation_status_str_to_int
-from ..exceptions import (SABaseException, SAExistingExportNameException,
-                          SANonExistingExportNameException)
-from .projects import get_project_metadata
+from ..common import annotation_status_str_to_int, upload_state_int_to_str
+from ..exceptions import (
+    SABaseException, SAExistingExportNameException,
+    SANonExistingExportNameException
+)
+from .project_api import get_project_metadata_bare
+from ..mixp.decorators import Trackable
 
 logger = logging.getLogger("superannotate-python-sdk")
 
@@ -26,8 +30,8 @@ _NUM_THREADS = 10
 def get_export_metadata(project, export_name):
     """Returns project metadata
 
-    :param project: project name or metadata of the project
-    :type project: str or dict
+    :param project: project name
+    :type project: str
     :param export_name: export name
     :type project: str
 
@@ -53,11 +57,12 @@ def get_export_metadata(project, export_name):
         )
 
 
+@Trackable
 def get_exports(project, return_metadata=False):
     """Get all prepared exports of the project.
 
-    :param project: project name or metadata of the project
-    :type project: str or dict
+    :param project: project name
+    :type project: str
     :param return_metadata: return metadata of images instead of names
     :type return_metadata: bool
 
@@ -65,7 +70,7 @@ def get_exports(project, return_metadata=False):
     :rtype: list of strs or dicts
     """
     if not isinstance(project, dict):
-        project = get_project_metadata(project)
+        project = get_project_metadata_bare(project)
     team_id, project_id = project["team_id"], project["id"]
     params = {'team_id': team_id, 'project_id': project_id}
     response = _api.send_request(req_type='GET', path='/exports', params=params)
@@ -94,14 +99,21 @@ def _get_export(export):
     return response.json()
 
 
+@Trackable
 def prepare_export(
-    project, annotation_statuses=None, include_fuse=False, only_pinned=False
+    project,
+    folder_names=None,
+    annotation_statuses=None,
+    include_fuse=False,
+    only_pinned=False
 ):
     """Prepare annotations and classes.json for export. Original and fused images for images with
     annotations can be included with include_fuse flag.
 
-    :param project: project name or metadata of the project
-    :type project: str or dict
+    :param project: project name
+    :type project: str
+    :param folder_names: names of folders to include in the export. If None, whole project will be exported
+    :type folder_names: list of str
     :param annotation_statuses: images with which status to include, if None, [ "InProgress", "QualityCheck", "Returned", "Completed"] will be chose
            list elements should be one of NotStarted InProgress QualityCheck Returned Completed Skipped
     :type annotation_statuses: list of strs
@@ -114,7 +126,22 @@ def prepare_export(
     :rtype: dict
     """
     if not isinstance(project, dict):
-        project = get_project_metadata(project)
+        project = get_project_metadata_bare(project)
+
+    upload_state = upload_state_int_to_str(project.get("upload_state"))
+
+    if upload_state == "External" and include_fuse:
+        logger.warning(
+            "Include fuse functionality is not supported for projects containing  items attached with URLs"
+        )
+        include_fuse = False
+    if project["type"] == "Video":
+        if only_pinned:
+            logger.warning(
+                "Pin functionality is not supported for projects containing videos attached with URLs"
+            )
+        only_pinned, include_fuse = False, False
+
     team_id, project_id = project["team_id"], project["id"]
     if annotation_statuses is None:
         annotation_statuses = [2, 3, 4, 5]
@@ -126,11 +153,13 @@ def prepare_export(
     current_time = datetime.now().strftime("%b %d %Y %H:%M")
     json_req = {
         "include": annotation_statuses,
-        "fuse": int(include_fuse),
-        "is_pinned": int(only_pinned),
         "coco": 0,
+        "is_pinned": int(only_pinned),
+        "fuse": int(include_fuse),
         "time": current_time
     }
+    if folder_names is not None:
+        json_req["folder_names"] = folder_names
     params = {'team_id': team_id, 'project_id': project_id}
     response = _api.send_request(
         req_type='POST', path='/export', params=params, json_req=json_req
@@ -140,10 +169,12 @@ def prepare_export(
             response.status_code, "Couldn't create_export." + response.text
         )
     res = response.json()
+    folder_str = "" if folder_names is None else ("/" + str(folder_names))
     logger.info(
-        "Prepared export %s for project %s.", res['name'], project["name"]
+        "Prepared export %s for project %s%s (project ID %s).", res['name'],
+        project["name"], folder_str, project["id"]
     )
-    return res["name"]
+    return res
 
 
 def __tqdm_thread(total_num, current_nums, finish_event):
@@ -181,7 +212,7 @@ def __upload_files_to_aws_thread(
         file = filepaths[i]
         try:
             relative_filename = file.relative_to(tmpdirname)
-            s3_key = f'{folder_path}/{relative_filename}'
+            s3_key = f'{folder_path}/{relative_filename.as_posix()}'
             to_s3.upload_file(str(file), s3_key)
         except Exception as e:
             logger.warning("Unable to upload to data server %s", e)
@@ -191,6 +222,16 @@ def __upload_files_to_aws_thread(
             already_uploaded[i] = True
 
 
+def _download_file(url, local_filename):
+    with requests.get(url, stream=True) as r:
+        r.raise_for_status()
+        with open(local_filename, 'wb') as f:
+            for chunk in r.iter_content(chunk_size=8192):
+                f.write(chunk)
+    return local_filename
+
+
+@Trackable
 def download_export(
     project, export, folder_path, extract_zip_contents=True, to_s3_bucket=None
 ):
@@ -199,10 +240,10 @@ def download_export(
     WARNING: Starting from version 1.9.0 :ref:`download_export <ref_download_export>` additionally
     requires :py:obj:`project` as first argument.
 
-    :param project: project name or metadata of the project
-    :type project: str or dict
-    :param export: export name or metadata of the prepared export
-    :type export: str or dict
+    :param project: project name
+    :type project: str
+    :param export: export name
+    :type export: str
     :param folder_path: where to download the export
     :type folder_path: Pathlike (str or Path)
     :param extract_zip_contents: if False then a zip file will be downloaded,
@@ -225,27 +266,28 @@ def download_export(
         break
 
     filename = Path(res['path']).name
-    r = requests.get(res['download'], allow_redirects=True)
-    if to_s3_bucket is None:
-        filepath = Path(folder_path) / filename
-        open(filepath, 'wb').write(r.content)
-        if extract_zip_contents:
-            with zipfile.ZipFile(filepath, 'r') as f:
-                f.extractall(folder_path)
-            Path.unlink(filepath)
-            logger.info("Extracted %s to folder %s", filepath, folder_path)
-        else:
-            logger.info("Downloaded export ID %s to %s", res['id'], filepath)
-    else:
-        with tempfile.TemporaryDirectory() as tmpdirname:
-            filepath = Path(tmpdirname) / filename
-            open(filepath, 'wb').write(r.content)
+    with tempfile.TemporaryDirectory() as tmpdirname:
+        temp_filepath = Path(tmpdirname) / filename
+        _download_file(res['download'], temp_filepath)
+        if to_s3_bucket is None:
+            filepath = Path(folder_path) / filename
+            shutil.copyfile(temp_filepath, filepath)
             if extract_zip_contents:
                 with zipfile.ZipFile(filepath, 'r') as f:
-                    f.extractall(tmpdirname)
+                    f.extractall(folder_path)
                 Path.unlink(filepath)
+                logger.info("Extracted %s to folder %s", filepath, folder_path)
+            else:
+                logger.info(
+                    "Downloaded export ID %s to %s", res['id'], filepath
+                )
+        else:
+            if extract_zip_contents:
+                with zipfile.ZipFile(temp_filepath, 'r') as f:
+                    f.extractall(tmpdirname)
+                Path.unlink(temp_filepath)
             files_to_upload = []
-            for file in Path(tmpdirname).rglob("*"):
+            for file in Path(tmpdirname).rglob("*.*"):
                 if not file.is_file():
                     continue
                 files_to_upload.append(file)
@@ -278,4 +320,4 @@ def download_export(
                     t.join()
             finish_event.set()
             tqdm_thread.join()
-        logger.info("Exported to AWS %s/%s", to_s3_bucket, folder_path)
+            logger.info("Exported to AWS %s/%s", to_s3_bucket, folder_path)
