@@ -1,40 +1,13 @@
 import json
 
-from .supervisely_converter import SuperviselyConverter
-from .supervisely_to_sa_vector import (
-    supervisely_to_sa, supervisely_instance_segmentation_to_sa_vector,
-    supervisely_object_detection_to_sa_vector,
-    supervisely_keypoint_detection_to_sa_vector
-)
-from .supervisely_to_sa_pixel import supervisely_instance_segmentation_to_sa_pixel
+from ..baseStrategy import baseStrategy
 
-from ....common import dump_output
+from ....common import write_to_json
 
 
-class SuperviselyObjectDetectionStrategy(SuperviselyConverter):
-    name = "ObjectDetection converter"
-
+class SuperviselyStrategy(baseStrategy):
     def __init__(self, args):
         super().__init__(args)
-        self.__setup_conversion_algorithm()
-
-    def __setup_conversion_algorithm(self):
-        if self.direction == "from":
-            if self.project_type == "Vector":
-                if self.task == 'vector_annotation':
-                    self.conversion_algorithm = supervisely_to_sa
-                elif self.task == 'object_detection':
-                    self.conversion_algorithm = supervisely_object_detection_to_sa_vector
-                elif self.task == 'instance_segmentation':
-                    self.conversion_algorithm = supervisely_instance_segmentation_to_sa_vector
-                elif self.task == 'keypoint_detection':
-                    self.conversion_algorithm = supervisely_keypoint_detection_to_sa_vector
-            elif self.project_type == "Pixel":
-                if self.task == 'instance_segmentation':
-                    self.conversion_algorithm = supervisely_instance_segmentation_to_sa_pixel
-
-    def __str__(self):
-        return '{} object'.format(self.name)
 
     def to_sa_format(self):
         sa_classes, classes_id_map = self._create_sa_classes()
@@ -50,15 +23,18 @@ class SuperviselyObjectDetectionStrategy(SuperviselyConverter):
         if self.conversion_algorithm.__name__ == 'supervisely_keypoint_detection_to_sa_vector':
             meta_json = json.load(open(self.export_root / 'meta.json'))
             sa_jsons = self.conversion_algorithm(
-                json_files, classes_id_map, meta_json
+                json_files, classes_id_map, meta_json, self.output_dir
             )
         elif self.conversion_algorithm.__name__ == 'supervisely_instance_segmentation_to_sa_pixel':
             sa_jsons = self.conversion_algorithm(
                 json_files, classes_id_map, self.output_dir
             )
         else:
-            sa_jsons = self.conversion_algorithm(json_files, classes_id_map)
-        dump_output(self.output_dir, self.platform, sa_classes, sa_jsons)
+            sa_jsons = self.conversion_algorithm(
+                json_files, classes_id_map, self.task, self.output_dir
+            )
+        (self.output_dir / 'classes').mkdir(exist_ok=True)
+        write_to_json(self.output_dir / 'classes' / 'classes.json', sa_classes)
 
     def _create_sa_classes(self):
         classes_json = json.load(open(self.export_root / 'meta.json'))
