@@ -3,10 +3,8 @@ Module which will convert from
 superannotate annotation format to other annotation formats
 """
 import copy
-import glob
 import json
 import logging
-import os
 import shutil
 
 from pathlib import Path
@@ -25,24 +23,26 @@ def _load_files(path_to_imgs, task, ptype):
     else:
         suffix = '___objects.json'
 
-    orig_images = glob.glob(os.path.join(path_to_imgs, '*'))
-    orig_images = [
-        x for x in orig_images if not os.path.isdir(x) and
-        '___' not in x.split('.')[-2] and "mapper.json" not in x
-    ]
+    orig_images = path_to_imgs.glob(('*' + suffix))
+    orig_images = [str(x).replace(suffix, '') for x in orig_images]
     all_files = None
-    if task == 'keypoint_detection':
-        all_files = np.array([(fname, fname + suffix) for fname in orig_images])
 
+    if task == 'keypoint_detection':
+        all_files = np.array([[fname, fname + suffix] for fname in orig_images])
     elif ptype == 'Pixel':
         all_files = np.array(
             [
-                (fname, fname + suffix, fname + '___save.png')
+                [fname, fname + suffix, fname + '___save.png']
                 for fname in orig_images
             ]
         )
     elif ptype == 'Vector':
-        all_files = np.array([(fname, fname + suffix) for fname in orig_images])
+        all_files = np.array(
+            [
+                [fname, fname + suffix, fname + '___fuse.png']
+                for fname in orig_images
+            ]
+        )
 
     return all_files
 
@@ -52,7 +52,8 @@ def _move_files(data_set, src):
     if data_set is not None:
         for tup in data_set:
             for i in tup:
-                shutil.copy(i, train_path / Path(i).name)
+                if Path(i).exists():
+                    shutil.copy(i, train_path / Path(i).name)
     else:
         logger.warning("Images doesn't exist")
 
@@ -60,8 +61,8 @@ def _move_files(data_set, src):
 def _create_classes_mapper(imgs, classes_json):
     classes = {}
     j_data = json.load(open(classes_json))
-    for i, instance in enumerate(j_data):
-        classes[instance['name']] = i + 1
+    for instance in j_data:
+        classes[instance['name']] = instance['id']
 
     with open(imgs / 'image_set' / 'classes_mapper.json', 'w') as fp:
         json.dump(classes, fp)
