@@ -8,39 +8,43 @@ import logging
 import os
 import time
 
-import boto3
-import pandas as pd
-import plotly.express as px
 import plotly.graph_objects as go
 from plotly.subplots import make_subplots
 
 from ..api import API
 from ..common import (
     _AVAILABLE_SEGMENTATION_MODELS, model_training_status_int_to_str,
-    process_api_response, project_type_str_to_int
+    project_type_str_to_int, upload_state_int_to_str, _MODEL_TRAINING_TASKS
 )
-from ..db.images import get_image_metadata, search_images
+from ..db.images import get_image_metadata
 from ..exceptions import SABaseException
 from ..parameter_decorators import model_metadata, project_metadata
 from .defaults import DEFAULT_HYPERPARAMETERS, NON_PLOTABLE_KEYS
 from .utils import log_process, make_plotly_specs, reformat_metrics_json
+from ..db.utils import _get_boto_session_by_credentials
+from ..db.project_api import get_project_metadata_bare, get_folder_metadata
+from ..mixp.decorators import Trackable
+from ..db.images import get_project_root_folder_id
+from .ml_models import search_models
 
 logger = logging.getLogger("superannotate-python-sdk")
 _api = API.get_instance()
 
 
+@Trackable
 @project_metadata
 @model_metadata
 def run_prediction(project, images_list, model):
     """This function runs smart prediction on given list of images from a given project using the neural network of your choice
+
     :param project: the project in which the target images are uploaded.
     :type project: str or dict
     :param images_list: the list of image names on which smart prediction has to be run
     :type images_list: list of str
     :param model: the name of the model that should be used for running smart prediction
     :type model: str or dict
-    :out res: tupe of two lists, list of images on which the prediction has succeded and failed respectively
-    :rtype res: tuple
+    :return: tupe of two lists, list of images on which the prediction has succeded and failed respectively
+    :rtype: tuple
     """
 
     if not isinstance(project, dict):
@@ -56,7 +60,12 @@ def run_prediction(project, images_list, model):
             f"Specified project has type {project['type']}, and does not correspond to the type of provided model"
         )
     project_id = project["id"]
-
+    upload_state = upload_state_int_to_str(project.get("upload_state"))
+    if upload_state == "External":
+        raise SABaseException(
+            0,
+            "The function does not support projects containing images attached with URLs"
+        )
     images_metadata = get_image_metadata(project, images_list)
     if isinstance(images_metadata, dict):
         images_metadata = [images_metadata]
@@ -100,6 +109,7 @@ def run_prediction(project, images_list, model):
     return succeded_imgs, failed_imgs
 
 
+@Trackable
 @project_metadata
 def run_segmentation(project, images_list, model):
     """Starts smart segmentation on a list of images using the specified model
@@ -108,7 +118,7 @@ def run_segmentation(project, images_list, model):
     :type  project: str or dict
     :param model  : The model name or metadata of the model
     :type  model  : str or dict
-    :out res: tupe of two lists, list of images on which the prediction has succeded and failed respectively
+    :return: tupe of two lists, list of images on which the prediction has succeded and failed respectively
     :rtype res: tuple
     """
 
@@ -126,6 +136,13 @@ def run_segmentation(project, images_list, model):
         )
         raise SABaseException(0, "Model Does not exist")
 
+    upload_state = upload_state_int_to_str(project.get("upload_state"))
+    if upload_state == "External":
+        raise SABaseException(
+            0,
+            "The function does not support projects containing images attached with URLs"
+        )
+
     images_metadata = get_image_metadata(project, images_list)
     images_metadata.sort(key=lambda x: x["name"])
 
@@ -168,56 +185,115 @@ def run_segmentation(project, images_list, model):
     return (succeded_imgs, failed_imgs)
 
 
-@project_metadata
-@model_metadata
+def _path_to_folder_id_project(path):
+    parts = path.split('/')
+    folder_id = None
+    project = None
+    if len(parts) == 1:
+        project_name = parts[0]
+        project = get_project_metadata_bare(project_name)
+        folder_id = get_project_root_folder_id(project)
+    elif len(parts) == 2:
+        project_name, folder_name = parts
+        project = get_project_metadata_bare(project_name)
+        folder = get_folder_metadata(project=project_name, folder_name=folder_name)
+        folder_id = folder['id']
+    return folder_id, project
+
+
+def _get_completed_images_counts(project_ids):
+    params = {
+        "team_id": _api.team_id,
+        "completedImagesCount": True
+    }
+    response = _api.send_request(
+        req_type="PUT",
+        path="/foldersByTeam",
+        json_req={"project_ids": project_ids},
+        params=params
+    )
+    return response.json()
+
+
+@Trackable
 def run_training(
-    project,
-    base_model,
-    model_name,
-    model_description,
-    task,
-    hyperparameters,
-    log=False
+        model_name,
+        model_description,
+        task,
+        base_model,
+        train_data,
+        test_data,
+        hyperparameters=None,
+        log=False
 ):
     """Runs neural network training
-    :param project: project or list of projects that contain the training images
-    :type  project: str, dict or list of dict
-    :param base_model  : base model on which the new network will be trained
-    :type  base_model  : str or dict
-    :param model_name   : name of the new model
-    :type  model_name   : str
+    :param model_name: name of the new model
+    :type  model_name: str
     :param model_description: description of the new model
     :type  model_description: str
-    :param task   : The model training task
-    :type  task   : str
-    :param hyperparameters: hyperparameters that should be used in training
+    :param task: The model training task
+    :type  task: str
+    :param base_model: base model on which the new network will be trained
+    :type  base_model: str or dict
+    :param train_data: train data folders (e.g., "project1/folder1")
+    :type  train_data: list of str
+    :param test_data: test data folders (e.g., "project1/folder1")
+    :type  test_data: list of str
+    :param hyperparameters: hyperparameters that should be used in training. If None use defualt hyperparameters for the training.
     :type  hyperparameters: dict
     :param log: If true will log training metrics in the stdout
     :type log: boolean
-    :out new_model: the metadata of the newly created model
-    :rtype new_model: dict
+    :return: the metadata of the newly created model
+    :rtype: dict
     """
 
-    project_ids = None
-    project_type = None
+    train_folder_ids = []
+    test_folder_ids = []
+    projects = []
 
-    if isinstance(project, dict):
-        project_ids = [project["id"]]
-        project_type = project["type"]
-        project = [project]
-    else:
-        project_ids = [x["id"] for x in project]
-        types = (x["type"] for x in project)
-        types = set(types)
-        if len(types) != 1:
-            logger.error(
-                "All projects have to be of the same type. Either vector or pixel"
-            )
-            raise SABaseException(0, "Invalid project types")
-        project_type = types.pop()
+    for path in train_data:
+        folder_id, project = _path_to_folder_id_project(path)
+        train_folder_ids.append(folder_id)
+        projects.append(project)
 
-    base_model = base_model.get(project_type, None)
-    if not base_model:
+    for path in test_data:
+        folder_id, project = _path_to_folder_id_project(path)
+        test_folder_ids.append(folder_id)
+        projects.append(project)
+
+    if set(train_folder_ids) & set(test_folder_ids):
+        raise SABaseException(
+            0,
+            "Avoid overlapping between training and test data."
+        )
+
+    types = [i["type"] for i in projects]
+    if len(set(types)) != 1:
+        logger.error(
+            "All projects have to be of the same type. Either vector or pixel"
+        )
+        raise SABaseException(0, "Invalid project types")
+
+    upload_states = set(i["upload_state"] for i in projects)
+    if any([True for state in upload_states if "External" == upload_state_int_to_str(state)]):
+        raise SABaseException(
+            0,
+            "The function does not support projects containing images attached with URLs"
+        )
+    if isinstance(base_model, dict):
+        base_model = base_model['name']
+    models = search_models(
+        include_global=True, name=base_model,type_= project_type_str_to_int(types[0]), task=_MODEL_TRAINING_TASKS[task]
+    )
+    if not models:
+        raise SABaseException(
+            0,
+            "The specifed model does not exist."
+        )
+    base_model = models[0]
+    base_model_id = base_model['id']
+    project_type = types[0]
+    if not base_model['type'] == project_type:
         logger.error(
             "The base model has to be of the same type (vector or pixel) as the projects"
         )
@@ -226,24 +302,30 @@ def run_training(
             f"The type of provided projects is {project_type}, and does not correspond to the type of provided model"
         )
 
-    for item in DEFAULT_HYPERPARAMETERS:
-        if item not in hyperparameters:
-            hyperparameters[item] = DEFAULT_HYPERPARAMETERS[item]
-    complete_image_count = 0
-    for proj in project:
-        complete_image_count += proj['completedImagesCount']
+    if not hyperparameters:
+        hyperparameters = DEFAULT_HYPERPARAMETERS
+    else:
+        for item in DEFAULT_HYPERPARAMETERS:
+            if item not in hyperparameters:
+                hyperparameters[item] = DEFAULT_HYPERPARAMETERS[item]
 
+    project_ids = [i['id'] for i in projects]
+    completed_images_data = _get_completed_images_counts(project_ids)
+    complete_image_count = 0
+    for folder in completed_images_data['data']:
+        if folder['id'] in train_folder_ids:
+            complete_image_count += folder['completedCount']
     hyperparameters["name"] = model_name
     hyperparameters["description"] = model_description
-    hyperparameters["task"] = task
-    hyperparameters["base_model_id"] = base_model["id"]
-    hyperparameters["project_ids"] = project_ids
+    hyperparameters["task"] = _MODEL_TRAINING_TASKS[task]
+    hyperparameters["base_model_id"] = base_model_id
     hyperparameters["image_count"] = complete_image_count
     hyperparameters["project_type"] = project_type_str_to_int(project_type)
+    hyperparameters["test_folder_ids"] = test_folder_ids
+    hyperparameters["train_folder_ids"] = train_folder_ids
     params = {
         "team_id": _api.team_id,
     }
-
     response = _api.send_request(
         req_type="POST",
         path="/ml_models",
@@ -306,7 +388,7 @@ def run_training(
                     if answer in ['Y', 'y']:
                         params = {'team_id': _api.team_id}
                         json_req = {'training_status': 6}
-                        response = _api.send_request(
+                        _api.send_request(
                             req_type='PUT',
                             path=f'ml_model/{new_model_id}',
                             params=params,
@@ -315,7 +397,7 @@ def run_training(
                         logger.info("Model was successfully saved")
                         pass
                     else:
-                        delete_model(name)
+                        delete_model(model_name)
                         logger.info('The model was not saved')
                 is_training_finished = True
 
@@ -323,14 +405,16 @@ def run_training(
     return new_model
 
 
+@Trackable
 @model_metadata
 def stop_model_training(model):
-    '''This function will stop training model provided by either name or metadata, and return the ID
+    """This function will stop training model provided by either name or metadata, and return the ID
+
     :param model: The name or the metadata of the model the training of which the user needs to terminate
-    :type model: str or dict
-    :out model: the metadata of the now, stopped model
-    :rtype out: dict
-    '''
+    :type model: dict
+    :return: the metadata of the now, stopped model
+    :rtype: dict
+    """
     params = {"team_id": _api.team_id}
     response = _api.send_request(
         req_type="POST",
@@ -345,11 +429,14 @@ def stop_model_training(model):
     return model
 
 
+@Trackable
 def plot_model_metrics(metric_json_list):
     """plots the metrics generated by neural network using plotly
+
        :param metric_json_list: list of <model_name>.json files
        :type  metric_json_list: list of str
     """
+
     def plot_df(df, plottable_cols, figure, start_index=1):
         for row, metric in enumerate(plottable_cols, start_index):
             for model_df in df:
@@ -371,7 +458,7 @@ def plot_model_metrics(metric_json_list):
             plottable_cols += [
                 col_name
                 for col_name in col_names if col_name not in plottable_cols and
-                col_name not in NON_PLOTABLE_KEYS
+                                             col_name not in NON_PLOTABLE_KEYS
             ]
         return plottable_cols
 
@@ -414,17 +501,20 @@ def plot_model_metrics(metric_json_list):
     figure.show()
 
 
+@Trackable
 @model_metadata
 def download_model(model, output_dir):
     """Downloads the neural network and related files
     which are the <model_name>.pth/pkl. <model_name>.json, <model_name>.yaml, classes_mapper.json
+
     :param model: the model that needs to be downloaded
     :type  model: str
     :param output_dir: the directiory in which the files will be saved
     :type output_dir: str
-    :out model: the metadata of the model that was deleted
-    :rtype model: dict
+    :return: the metadata of the model
+    :rtype: dict
     """
+
     if not os.path.isdir(output_dir):
         os.makedirs(output_dir, exist_ok=True)
 
@@ -449,16 +539,12 @@ def download_model(model, output_dir):
         params=params
     )
     if response.ok:
-        response = process_api_response(response.json())
+        response = response.json()
     else:
         raise SABaseException(0, "Could not get model info ")
 
     tokens = response["tokens"]
-    s3_session = boto3.Session(
-        aws_access_key_id=tokens["accessKeyId"],
-        aws_secret_access_key=tokens["secretAccessKey"],
-        aws_session_token=tokens["sessionToken"]
-    )
+    s3_session = _get_boto_session_by_credentials(tokens)
     s3_resource = s3_session.resource('s3')
 
     bucket = s3_resource.Bucket(tokens["bucket"])
@@ -480,13 +566,16 @@ def download_model(model, output_dir):
     return model
 
 
+@Trackable
 @model_metadata
 def delete_model(model):
-    '''This function deletes the provided model
+    """This function deletes the provided model
+
        :param model: the model to be deleted
-       :out model: the metadata of the model that was deleted
-       :rtype out: dict
-    '''
+       :type model: dict
+       :return: the metadata of the model that was deleted
+       :rtype: dict
+    """
     params = {"team_id": _api.team_id}
     response = _api.send_request(
         req_type="DELETE", path=f'/ml_model/{model["id"]}', params=params
