@@ -1,10 +1,11 @@
-from ..common import process_api_response
-from ..exceptions import SABaseException
-from ..api import API
-from ..common import project_type_int_to_str
 import logging
 
+from ..api import API
+from ..common import project_type_int_to_str
+from ..exceptions import SABaseException
+
 logger = logging.getLogger("superannotate-python-sdk")
+
 _api = API.get_instance()
 
 
@@ -15,6 +16,22 @@ def search_models(
     task=None,
     include_global=True,
 ):
+    """Search for ML models.
+
+    :param name: search string
+    :type name: str
+    :param type_: ml model type string
+    :type type_: str
+    :param project_id: project id
+    :type project_id: str
+    :param task: training task
+    :type task: str
+    :param include_global: include global ml models
+    :type include_global: bool
+
+    :return: ml model metadata
+    :rtype: list of dicts
+    """
     params = {
         "name": name,
         "team_id": _api.team_id,
@@ -30,7 +47,7 @@ def search_models(
 
     if not response.ok:
         raise SABaseException(0, "could not search models")
-    result = process_api_response(response.json())
+    result = response.json()
 
     for model in result['data']:
         model['type'] = project_type_int_to_str(model['type'])
